help_text = {
    # Index screen
    'experimental_runs':'Design an experiment that runs different parameter configurations by specifying parameter '
                        'bounds.\nThe results can be visualised with parallel coordinates, scatter plots and heatmaps.',
    'standard_runs':'Help1',
    'import_results':'Help2',

    # Import screen
    'import_experiment':'Help3',
    'import_standard':'Help4',
    'import_behaviorspace':'Help5',

    # Experiment/Standard screen
    'run_repetitions': 'Help6',
    'run_ticks': 'Help7',
    'run_reporters': 'Help8',
    'run_setup': 'Help9',
    'run_processes': 'Help10',
    'save_config':'Help11',

    # Experiment screen
    'experiment_variables':'Help12',
    'experiment_sampling':'Help13',
    'experiment_scenarios':'Help14',

    # Standard screen
    'standard_variables':'Help15',

    # Experiment/Standard plots
    'config_information':'Help',
    'save_results':'Help',

    # Experiment plots
    'parallel_coordinates':'Help',
    'scatterplot':'Help',
    'scatterplot_variables':'Help',
    'heatmap':'Help',
    '4_metric':'Help',

    # Standard plots
    'timeseries':'Help',
    'boxplot':'Help',
    'violinplot':'Help',
    'histogram':'Help',
    'distributionplot':'Help',
    'distributionplot_indicators':'Help',
    '3_metric':'Help',
    'standard_plot_reporters':'Help',

}
